/****************************************************************************
** Meta object code from reading C++ file 'qibusconfigproxy.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/ibus-qt-1.3.3/src/qibusconfigproxy.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qibusconfigproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_IBusConfigProxy[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: signature, parameters, type, tag, flags
      36,   17,   16,   16, 0x05,

 // slots: signature, parameters, type, tag, flags
      99,   16,   79,   16, 0x0a,
     154,  141,  109,   16, 0x0a,
     180,   17,   79,   16, 0x0a,
     219,  141,   79,   16, 0x0a,

       0        // eod
};

static const char qt_meta_stringdata_IBusConfigProxy[] = {
    "IBusConfigProxy\0\0section,name,value\0"
    "ValueChanged(QString,QString,QDBusVariant)\0"
    "QDBusPendingReply<>\0Destroy()\0"
    "QDBusPendingReply<QDBusVariant>\0"
    "section,name\0GetValue(QString,QString)\0"
    "SetValue(QString,QString,QDBusVariant)\0"
    "Unset(QString,QString)\0"
};

void IBusConfigProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        IBusConfigProxy *_t = static_cast<IBusConfigProxy *>(_o);
        switch (_id) {
        case 0: _t->ValueChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QDBusVariant(*)>(_a[3]))); break;
        case 1: { QDBusPendingReply<> _r = _t->Destroy();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 2: { QDBusPendingReply<QDBusVariant> _r = _t->GetValue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QDBusVariant>*>(_a[0]) = _r; }  break;
        case 3: { QDBusPendingReply<> _r = _t->SetValue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QDBusVariant(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 4: { QDBusPendingReply<> _r = _t->Unset((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        default: ;
        }
    }
}

const QMetaObjectExtraData IBusConfigProxy::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject IBusConfigProxy::staticMetaObject = {
    { &QDBusAbstractInterface::staticMetaObject, qt_meta_stringdata_IBusConfigProxy,
      qt_meta_data_IBusConfigProxy, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &IBusConfigProxy::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *IBusConfigProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *IBusConfigProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_IBusConfigProxy))
        return static_cast<void*>(const_cast< IBusConfigProxy*>(this));
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int IBusConfigProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void IBusConfigProxy::ValueChanged(const QString & _t1, const QString & _t2, const QDBusVariant & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
