/****************************************************************************
** Meta object code from reading C++ file 'qibusengine.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/ibus-qt-1.3.3/src/qibusengine.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qibusengine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_IBus__Engine[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
      34,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      17,       // signalCount

 // signals: signature, parameters, type, tag, flags
      19,   14,   13,   13, 0x05,
      44,   13,   13,   13, 0x05,
      68,   13,   13,   13, 0x05,
     111,   90,   13,   13, 0x05,
     143,   13,   13,   13, 0x05,
     163,   13,   13,   13, 0x05,
     181,   13,   13,   13, 0x05,
     199,   13,   13,   13, 0x05,
     221,   13,   13,   13, 0x05,
     247,  241,   13,   13, 0x05,
     280,   13,   13,   13, 0x05,
     300,   13,   13,   13, 0x05,
     318,   13,   13,   13, 0x05,
     349,  336,   13,   13, 0x05,
     409,  388,   13,   13, 0x05,
     475,  446,   13,   13, 0x05,
     527,  522,   13,   13, 0x05,

 // slots: signature, parameters, type, tag, flags
     575,  556,   13,   13, 0x08,
     608,   13,   13,   13, 0x08,
     621,   13,   13,   13, 0x08,
     632,   13,   13,   13, 0x08,
     642,   13,   13,   13, 0x08,
     652,   13,   13,   13, 0x08,
     661,   13,   13,   13, 0x08,
     671,   13,   13,   13, 0x08,
     682,   13,   13,   13, 0x08,
     693,   13,   13,   13, 0x08,
     707,   90,  702,   13, 0x08,
     760,  739,   13,   13, 0x08,
     800,  790,   13,   13, 0x08,
     822,  790,   13,   13, 0x08,
     844,   13,   13,   13, 0x08,
     856,  852,   13,   13, 0x08,
     886,  878,   13,   13, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_IBus__Engine[] = {
    "IBus::Engine\0\0text\0CommitText(QDBusVariant)\0"
    "CursorDownLookupTable()\0CursorUpLookupTable()\0"
    "keyval,keycode,state\0"
    "ForwardKeyEvent(uint,uint,uint)\0"
    "HideAuxiliaryText()\0HideLookupTable()\0"
    "HidePreeditText()\0PageDownLookupTable()\0"
    "PageUpLookupTable()\0props\0"
    "RegisterProperties(QDBusVariant)\0"
    "ShowAuxiliaryText()\0ShowLookupTable()\0"
    "ShowPreeditText()\0text,visible\0"
    "UpdateAuxiliaryText(QDBusVariant,bool)\0"
    "lookup_table,visible\0"
    "UpdateLookupTable(QDBusVariant,bool)\0"
    "text,cursor_pos,visible,mode\0"
    "UpdatePreeditText(QDBusVariant,uint,bool,uint)\0"
    "prop\0UpdateProperty(QDBusVariant)\0"
    "index,button,state\0CandidateClicked(uint,uint,uint)\0"
    "CursorDown()\0CursorUp()\0Destroy()\0"
    "Disable()\0Enable()\0FocusIn()\0FocusOut()\0"
    "PageDown()\0PageUp()\0bool\0"
    "ProcessKeyEvent(uint,uint,uint)\0"
    "prop_name,prop_state\0PropertyActivate(QString,int)\0"
    "prop_name\0PropertyHide(QString)\0"
    "PropertyShow(QString)\0Reset()\0cap\0"
    "SetCapabilities(uint)\0x,y,w,h\0"
    "SetCursorLocation(int,int,int,int)\0"
};

void IBus::Engine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        Engine *_t = static_cast<Engine *>(_o);
        switch (_id) {
        case 0: _t->CommitText((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 1: _t->CursorDownLookupTable(); break;
        case 2: _t->CursorUpLookupTable(); break;
        case 3: _t->ForwardKeyEvent((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3]))); break;
        case 4: _t->HideAuxiliaryText(); break;
        case 5: _t->HideLookupTable(); break;
        case 6: _t->HidePreeditText(); break;
        case 7: _t->PageDownLookupTable(); break;
        case 8: _t->PageUpLookupTable(); break;
        case 9: _t->RegisterProperties((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 10: _t->ShowAuxiliaryText(); break;
        case 11: _t->ShowLookupTable(); break;
        case 12: _t->ShowPreeditText(); break;
        case 13: _t->UpdateAuxiliaryText((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 14: _t->UpdateLookupTable((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 15: _t->UpdatePreeditText((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< uint(*)>(_a[4]))); break;
        case 16: _t->UpdateProperty((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 17: _t->CandidateClicked((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3]))); break;
        case 18: _t->CursorDown(); break;
        case 19: _t->CursorUp(); break;
        case 20: _t->Destroy(); break;
        case 21: _t->Disable(); break;
        case 22: _t->Enable(); break;
        case 23: _t->FocusIn(); break;
        case 24: _t->FocusOut(); break;
        case 25: _t->PageDown(); break;
        case 26: _t->PageUp(); break;
        case 27: { bool _r = _t->ProcessKeyEvent((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 28: _t->PropertyActivate((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 29: _t->PropertyHide((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 30: _t->PropertyShow((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 31: _t->Reset(); break;
        case 32: _t->SetCapabilities((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 33: _t->SetCursorLocation((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData IBus::Engine::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject IBus::Engine::staticMetaObject = {
    { &Object::staticMetaObject, qt_meta_stringdata_IBus__Engine,
      qt_meta_data_IBus__Engine, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &IBus::Engine::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *IBus::Engine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *IBus::Engine::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_IBus__Engine))
        return static_cast<void*>(const_cast< Engine*>(this));
    return Object::qt_metacast(_clname);
}

int IBus::Engine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Object::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    return _id;
}

// SIGNAL 0
void IBus::Engine::CommitText(const QDBusVariant & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void IBus::Engine::CursorDownLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 1, 0);
}

// SIGNAL 2
void IBus::Engine::CursorUpLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 2, 0);
}

// SIGNAL 3
void IBus::Engine::ForwardKeyEvent(uint _t1, uint _t2, uint _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void IBus::Engine::HideAuxiliaryText()
{
    QMetaObject::activate(this, &staticMetaObject, 4, 0);
}

// SIGNAL 5
void IBus::Engine::HideLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 5, 0);
}

// SIGNAL 6
void IBus::Engine::HidePreeditText()
{
    QMetaObject::activate(this, &staticMetaObject, 6, 0);
}

// SIGNAL 7
void IBus::Engine::PageDownLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 7, 0);
}

// SIGNAL 8
void IBus::Engine::PageUpLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 8, 0);
}

// SIGNAL 9
void IBus::Engine::RegisterProperties(const QDBusVariant & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void IBus::Engine::ShowAuxiliaryText()
{
    QMetaObject::activate(this, &staticMetaObject, 10, 0);
}

// SIGNAL 11
void IBus::Engine::ShowLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 11, 0);
}

// SIGNAL 12
void IBus::Engine::ShowPreeditText()
{
    QMetaObject::activate(this, &staticMetaObject, 12, 0);
}

// SIGNAL 13
void IBus::Engine::UpdateAuxiliaryText(const QDBusVariant & _t1, bool _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void IBus::Engine::UpdateLookupTable(const QDBusVariant & _t1, bool _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void IBus::Engine::UpdatePreeditText(const QDBusVariant & _t1, uint _t2, bool _t3, uint _t4)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)), const_cast<void*>(reinterpret_cast<const void*>(&_t4)) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void IBus::Engine::UpdateProperty(const QDBusVariant & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}
QT_END_MOC_NAMESPACE
