/****************************************************************************
** Meta object code from reading C++ file 'qibusconfigadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/ibus-qt-1.3.3/src/qibusconfigadaptor.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qibusconfigadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_IBusConfigAdaptor[] = {

 // content:
       6,       // revision
       0,       // classname
       2,   14, // classinfo
       5,   18, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
      46,   18,
     826,   62,

 // signals: signature, parameters, type, tag, flags
     866,  847,  846,  846, 0x05,

 // slots: signature, parameters, type, tag, flags
     909,  846,  846,  846, 0x0a,
     945,  932,  919,  846, 0x0a,
     971,  847,  846,  846, 0x0a,
    1010,  932,  846,  846, 0x0a,

       0        // eod
};

static const char qt_meta_stringdata_IBusConfigAdaptor[] = {
    "IBusConfigAdaptor\0org.freedesktop.IBus.Config\0"
    "D-Bus Interface\0"
    "  <interface name=\"org.freedesktop.IBus.Config\">\n    <method name=\""
    "Destroy\"/>\n    <signal name=\"ValueChanged\">\n      <arg type=\"s\""
    " name=\"section\"/>\n      <arg type=\"s\" name=\"name\"/>\n      <arg"
    " type=\"v\" name=\"value\"/>\n    </signal>\n    <method name=\"SetVal"
    "ue\">\n      <arg direction=\"in\" type=\"s\" name=\"section\"/>\n    "
    "  <arg direction=\"in\" type=\"s\" name=\"name\"/>\n      <arg directi"
    "on=\"in\" type=\"v\" name=\"value\"/>\n    </method>\n    <method name"
    "=\"GetValue\">\n      <arg direction=\"in\" type=\"s\" name=\"section\""
    "/>\n      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n      <arg"
    " direction=\"out\" type=\"v\"/>\n    </method>\n    <method name=\"Uns"
    "et\">\n      <arg direction=\"in\" type=\"s\" name=\"section\"/>\n    "
    "  <arg direction=\"in\" type=\"s\" name=\"name\"/>\n    </method>\n  <"
    "/interface>\n\0"
    "D-Bus Introspection\0\0section,name,value\0"
    "ValueChanged(QString,QString,QDBusVariant)\0"
    "Destroy()\0QDBusVariant\0section,name\0"
    "GetValue(QString,QString)\0"
    "SetValue(QString,QString,QDBusVariant)\0"
    "Unset(QString,QString)\0"
};

void IBusConfigAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        IBusConfigAdaptor *_t = static_cast<IBusConfigAdaptor *>(_o);
        switch (_id) {
        case 0: _t->ValueChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QDBusVariant(*)>(_a[3]))); break;
        case 1: _t->Destroy(); break;
        case 2: { QDBusVariant _r = _t->GetValue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = _r; }  break;
        case 3: _t->SetValue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QDBusVariant(*)>(_a[3]))); break;
        case 4: _t->Unset((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData IBusConfigAdaptor::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject IBusConfigAdaptor::staticMetaObject = {
    { &QDBusAbstractAdaptor::staticMetaObject, qt_meta_stringdata_IBusConfigAdaptor,
      qt_meta_data_IBusConfigAdaptor, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &IBusConfigAdaptor::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *IBusConfigAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *IBusConfigAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_IBusConfigAdaptor))
        return static_cast<void*>(const_cast< IBusConfigAdaptor*>(this));
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int IBusConfigAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void IBusConfigAdaptor::ValueChanged(const QString & _t1, const QString & _t2, const QDBusVariant & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
