/****************************************************************************
** Meta object code from reading C++ file 'qibusinputcontext.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/ibus-qt-1.3.3/src/qibusinputcontext.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qibusinputcontext.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_IBus__InputContext[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
      42,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      21,       // signalCount

 // signals: signature, parameters, type, tag, flags
      25,   20,   19,   19, 0x05,
      73,   49,   19,   19, 0x05,
     114,   19,   19,   19, 0x05,
     132,   19,   19,   19, 0x05,
     163,  150,   19,   19, 0x05,
     201,   19,   19,   19, 0x05,
     221,   19,   19,   19, 0x05,
     255,  241,   19,   19, 0x05,
     298,   19,   19,   19, 0x05,
     316,   19,   19,   19, 0x05,
     334,   19,   19,   19, 0x05,
     356,   19,   19,   19, 0x05,
     380,   19,   19,   19, 0x05,
     400,   19,   19,   19, 0x05,
     428,  422,   19,   19, 0x05,
     473,  464,   19,   19, 0x05,
     505,   19,   19,   19, 0x05,
     515,   19,   19,   19, 0x05,
     547,  526,   19,   19, 0x05,
     593,  579,   19,   19, 0x05,
     625,   19,   19,   19, 0x05,

 // slots: signature, parameters, type, tag, flags
     650,   20,   19,   19, 0x08,
     679,   49,   19,   19, 0x08,
     725,   19,   19,   19, 0x08,
     747,   19,   19,   19, 0x08,
     769,  150,   19,   19, 0x08,
     812,   19,   19,   19, 0x08,
     836,   19,   19,   19, 0x08,
     860,  241,   19,   19, 0x08,
     901,   19,   19,   19, 0x08,
     923,   19,   19,   19, 0x08,
     945,   19,   19,   19, 0x08,
     971,   19,   19,   19, 0x08,
     999,   19,   19,   19, 0x08,
    1023,   19,   19,   19, 0x08,
    1049,  422,   19,   19, 0x08,
    1091, 1086,   19,   19, 0x08,
    1124,   19,   19,   19, 0x08,
    1138,   19,   19,   19, 0x08,
    1153,  526,   19,   19, 0x08,
    1189,  579,   19,   19, 0x08,
    1225,   19,   19,   19, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_IBus__InputContext[] = {
    "IBus::InputContext\0\0text\0"
    "commitText(TextPointer)\0text,cursor_pos,visible\0"
    "updatePreeditText(TextPointer,uint,bool)\0"
    "showPreeditText()\0hidePreeditText()\0"
    "text,visible\0updateAuxiliaryText(TextPointer,bool)\0"
    "showAuxiliaryText()\0hideAuxiliaryText()\0"
    "table,visible\0updateLookupTable(LookupTablePointer,bool)\0"
    "showLookupTable()\0hideLookupTable()\0"
    "cursorUpLookupTable()\0cursorDownLookupTable()\0"
    "pageUpLookupTable()\0pageDownLookupTable()\0"
    "props\0registerProperties(PropListPointer)\0"
    "property\0updateProperty(PropertyPointer)\0"
    "enabled()\0disabled()\0keyval,keycode,state\0"
    "forwardKeyEvent(uint,uint,uint)\0"
    "offset,nchars\0deleteSurroundingText(int,uint)\0"
    "requireSurroundingText()\0"
    "slotCommitText(QDBusVariant)\0"
    "slotUpdatePreeditText(QDBusVariant,uint,bool)\0"
    "slotHidePreeditText()\0slotShowPreeditText()\0"
    "slotUpdateAuxiliaryText(QDBusVariant,bool)\0"
    "slotShowAuxiliaryText()\0slotHideAuxiliaryText()\0"
    "slotUpdateLookupTable(QDBusVariant,bool)\0"
    "slotShowLookupTable()\0slotHideLookupTable()\0"
    "slotCursorUpLookupTable()\0"
    "slotCursorDownLookupTable()\0"
    "slotPageUpLookupTable()\0"
    "slotPageDownLookupTable()\0"
    "slotRegisterProperties(QDBusVariant)\0"
    "prop\0slotUpdateProperty(QDBusVariant)\0"
    "slotEnabled()\0slotDisabled()\0"
    "slotForwardKeyEvent(uint,uint,uint)\0"
    "slotDeleteSurroundingText(int,uint)\0"
    "slotRequireSurroundingText()\0"
};

void IBus::InputContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        InputContext *_t = static_cast<InputContext *>(_o);
        switch (_id) {
        case 0: _t->commitText((*reinterpret_cast< const TextPointer(*)>(_a[1]))); break;
        case 1: _t->updatePreeditText((*reinterpret_cast< const TextPointer(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 2: _t->showPreeditText(); break;
        case 3: _t->hidePreeditText(); break;
        case 4: _t->updateAuxiliaryText((*reinterpret_cast< const TextPointer(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->showAuxiliaryText(); break;
        case 6: _t->hideAuxiliaryText(); break;
        case 7: _t->updateLookupTable((*reinterpret_cast< const LookupTablePointer(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 8: _t->showLookupTable(); break;
        case 9: _t->hideLookupTable(); break;
        case 10: _t->cursorUpLookupTable(); break;
        case 11: _t->cursorDownLookupTable(); break;
        case 12: _t->pageUpLookupTable(); break;
        case 13: _t->pageDownLookupTable(); break;
        case 14: _t->registerProperties((*reinterpret_cast< const PropListPointer(*)>(_a[1]))); break;
        case 15: _t->updateProperty((*reinterpret_cast< const PropertyPointer(*)>(_a[1]))); break;
        case 16: _t->enabled(); break;
        case 17: _t->disabled(); break;
        case 18: _t->forwardKeyEvent((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3]))); break;
        case 19: _t->deleteSurroundingText((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 20: _t->requireSurroundingText(); break;
        case 21: _t->slotCommitText((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 22: _t->slotUpdatePreeditText((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 23: _t->slotHidePreeditText(); break;
        case 24: _t->slotShowPreeditText(); break;
        case 25: _t->slotUpdateAuxiliaryText((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 26: _t->slotShowAuxiliaryText(); break;
        case 27: _t->slotHideAuxiliaryText(); break;
        case 28: _t->slotUpdateLookupTable((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 29: _t->slotShowLookupTable(); break;
        case 30: _t->slotHideLookupTable(); break;
        case 31: _t->slotCursorUpLookupTable(); break;
        case 32: _t->slotCursorDownLookupTable(); break;
        case 33: _t->slotPageUpLookupTable(); break;
        case 34: _t->slotPageDownLookupTable(); break;
        case 35: _t->slotRegisterProperties((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 36: _t->slotUpdateProperty((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 37: _t->slotEnabled(); break;
        case 38: _t->slotDisabled(); break;
        case 39: _t->slotForwardKeyEvent((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3]))); break;
        case 40: _t->slotDeleteSurroundingText((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 41: _t->slotRequireSurroundingText(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData IBus::InputContext::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject IBus::InputContext::staticMetaObject = {
    { &Object::staticMetaObject, qt_meta_stringdata_IBus__InputContext,
      qt_meta_data_IBus__InputContext, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &IBus::InputContext::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *IBus::InputContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *IBus::InputContext::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_IBus__InputContext))
        return static_cast<void*>(const_cast< InputContext*>(this));
    return Object::qt_metacast(_clname);
}

int IBus::InputContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Object::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
    return _id;
}

// SIGNAL 0
void IBus::InputContext::commitText(const TextPointer & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void IBus::InputContext::updatePreeditText(const TextPointer & _t1, uint _t2, bool _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void IBus::InputContext::showPreeditText()
{
    QMetaObject::activate(this, &staticMetaObject, 2, 0);
}

// SIGNAL 3
void IBus::InputContext::hidePreeditText()
{
    QMetaObject::activate(this, &staticMetaObject, 3, 0);
}

// SIGNAL 4
void IBus::InputContext::updateAuxiliaryText(const TextPointer & _t1, bool _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void IBus::InputContext::showAuxiliaryText()
{
    QMetaObject::activate(this, &staticMetaObject, 5, 0);
}

// SIGNAL 6
void IBus::InputContext::hideAuxiliaryText()
{
    QMetaObject::activate(this, &staticMetaObject, 6, 0);
}

// SIGNAL 7
void IBus::InputContext::updateLookupTable(const LookupTablePointer & _t1, bool _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void IBus::InputContext::showLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 8, 0);
}

// SIGNAL 9
void IBus::InputContext::hideLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 9, 0);
}

// SIGNAL 10
void IBus::InputContext::cursorUpLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 10, 0);
}

// SIGNAL 11
void IBus::InputContext::cursorDownLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 11, 0);
}

// SIGNAL 12
void IBus::InputContext::pageUpLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 12, 0);
}

// SIGNAL 13
void IBus::InputContext::pageDownLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 13, 0);
}

// SIGNAL 14
void IBus::InputContext::registerProperties(const PropListPointer & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void IBus::InputContext::updateProperty(const PropertyPointer & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void IBus::InputContext::enabled()
{
    QMetaObject::activate(this, &staticMetaObject, 16, 0);
}

// SIGNAL 17
void IBus::InputContext::disabled()
{
    QMetaObject::activate(this, &staticMetaObject, 17, 0);
}

// SIGNAL 18
void IBus::InputContext::forwardKeyEvent(uint _t1, uint _t2, uint _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void IBus::InputContext::deleteSurroundingText(int _t1, uint _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void IBus::InputContext::requireSurroundingText()
{
    QMetaObject::activate(this, &staticMetaObject, 20, 0);
}
QT_END_MOC_NAMESPACE
