/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -a qibusfactoryadaptor -c IBusFactoryAdaptor -v org.freedesktop.IBus.EngineFactory.xml org.freedesktop.IBus.EngineFactory
 *
 * qdbusxml2cpp is Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "qibusfactoryadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class IBusFactoryAdaptor
 */

IBusFactoryAdaptor::IBusFactoryAdaptor(QObject *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

IBusFactoryAdaptor::~IBusFactoryAdaptor()
{
    // destructor
}

QDBusObjectPath IBusFactoryAdaptor::CreateEngine(const QString &engine_name)
{
    // handle method call org.freedesktop.IBus.EngineFactory.CreateEngine
    QDBusObjectPath out0;
    QMetaObject::invokeMethod(parent(), "CreateEngine", Q_RETURN_ARG(QDBusObjectPath, out0), Q_ARG(QString, engine_name));
    return out0;
}

void IBusFactoryAdaptor::Destroy()
{
    // handle method call org.freedesktop.IBus.EngineFactory.Destroy
    QMetaObject::invokeMethod(parent(), "Destroy");
}

