/****************************************************************************
** Meta object code from reading C++ file 'qibusinputcontextproxy.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/ibus-qt-1.3.3/src/qibusinputcontextproxy.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qibusinputcontextproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_IBusInputContextProxy[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
      35,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      21,       // signalCount

 // signals: signature, parameters, type, tag, flags
      28,   23,   22,   22, 0x05,
      53,   22,   22,   22, 0x05,
      77,   22,   22,   22, 0x05,
     113,   99,   22,   22, 0x05,
     145,   22,   22,   22, 0x05,
     156,   22,   22,   22, 0x05,
     187,  166,   22,   22, 0x05,
     219,   22,   22,   22, 0x05,
     239,   22,   22,   22, 0x05,
     257,   22,   22,   22, 0x05,
     275,   22,   22,   22, 0x05,
     297,   22,   22,   22, 0x05,
     323,  317,   22,   22, 0x05,
     356,   22,   22,   22, 0x05,
     381,   22,   22,   22, 0x05,
     401,   22,   22,   22, 0x05,
     419,   22,   22,   22, 0x05,
     450,  437,   22,   22, 0x05,
     503,  489,   22,   22, 0x05,
     564,  540,   22,   22, 0x05,
     611,  606,   22,   22, 0x05,

 // slots: signature, parameters, type, tag, flags
     660,   22,  640,   22, 0x0a,
     670,   22,  640,   22, 0x0a,
     680,   22,  640,   22, 0x0a,
     689,   22,  640,   22, 0x0a,
     699,   22,  640,   22, 0x0a,
     742,   22,  710,   22, 0x0a,
     778,   22,  754,   22, 0x0a,
     790,  166,  754,   22, 0x0a,
     833,  822,  640,   22, 0x0a,
     863,   22,  640,   22, 0x0a,
     876,  871,  640,   22, 0x0a,
     906,  898,  640,   22, 0x0a,
     946,  941,  640,   22, 0x0a,
     992,  965,  640,   22, 0x0a,

       0        // eod
};

static const char qt_meta_stringdata_IBusInputContextProxy[] = {
    "IBusInputContextProxy\0\0text\0"
    "CommitText(QDBusVariant)\0"
    "CursorDownLookupTable()\0CursorUpLookupTable()\0"
    "offset,nchars\0DeleteSurroundingText(int,uint)\0"
    "Disabled()\0Enabled()\0keyval,keycode,state\0"
    "ForwardKeyEvent(uint,uint,uint)\0"
    "HideAuxiliaryText()\0HideLookupTable()\0"
    "HidePreeditText()\0PageDownLookupTable()\0"
    "PageUpLookupTable()\0props\0"
    "RegisterProperties(QDBusVariant)\0"
    "RequireSurroundingText()\0ShowAuxiliaryText()\0"
    "ShowLookupTable()\0ShowPreeditText()\0"
    "text,visible\0UpdateAuxiliaryText(QDBusVariant,bool)\0"
    "table,visible\0UpdateLookupTable(QDBusVariant,bool)\0"
    "text,cursor_pos,visible\0"
    "UpdatePreeditText(QDBusVariant,uint,bool)\0"
    "prop\0UpdateProperty(QDBusVariant)\0"
    "QDBusPendingReply<>\0Destroy()\0Disable()\0"
    "Enable()\0FocusIn()\0FocusOut()\0"
    "QDBusPendingReply<QDBusVariant>\0"
    "GetEngine()\0QDBusPendingReply<bool>\0"
    "IsEnabled()\0ProcessKeyEvent(uint,uint,uint)\0"
    "name,state\0PropertyActivate(QString,int)\0"
    "Reset()\0caps\0SetCapabilities(uint)\0"
    "x,y,w,h\0SetCursorLocation(int,int,int,int)\0"
    "name\0SetEngine(QString)\0"
    "text,cursor_pos,anchor_pos\0"
    "SetSurroundingText(QDBusVariant,uint,uint)\0"
};

void IBusInputContextProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        IBusInputContextProxy *_t = static_cast<IBusInputContextProxy *>(_o);
        switch (_id) {
        case 0: _t->CommitText((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 1: _t->CursorDownLookupTable(); break;
        case 2: _t->CursorUpLookupTable(); break;
        case 3: _t->DeleteSurroundingText((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 4: _t->Disabled(); break;
        case 5: _t->Enabled(); break;
        case 6: _t->ForwardKeyEvent((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3]))); break;
        case 7: _t->HideAuxiliaryText(); break;
        case 8: _t->HideLookupTable(); break;
        case 9: _t->HidePreeditText(); break;
        case 10: _t->PageDownLookupTable(); break;
        case 11: _t->PageUpLookupTable(); break;
        case 12: _t->RegisterProperties((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 13: _t->RequireSurroundingText(); break;
        case 14: _t->ShowAuxiliaryText(); break;
        case 15: _t->ShowLookupTable(); break;
        case 16: _t->ShowPreeditText(); break;
        case 17: _t->UpdateAuxiliaryText((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 18: _t->UpdateLookupTable((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 19: _t->UpdatePreeditText((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 20: _t->UpdateProperty((*reinterpret_cast< const QDBusVariant(*)>(_a[1]))); break;
        case 21: { QDBusPendingReply<> _r = _t->Destroy();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 22: { QDBusPendingReply<> _r = _t->Disable();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 23: { QDBusPendingReply<> _r = _t->Enable();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 24: { QDBusPendingReply<> _r = _t->FocusIn();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 25: { QDBusPendingReply<> _r = _t->FocusOut();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 26: { QDBusPendingReply<QDBusVariant> _r = _t->GetEngine();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QDBusVariant>*>(_a[0]) = _r; }  break;
        case 27: { QDBusPendingReply<bool> _r = _t->IsEnabled();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = _r; }  break;
        case 28: { QDBusPendingReply<bool> _r = _t->ProcessKeyEvent((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = _r; }  break;
        case 29: { QDBusPendingReply<> _r = _t->PropertyActivate((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 30: { QDBusPendingReply<> _r = _t->Reset();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 31: { QDBusPendingReply<> _r = _t->SetCapabilities((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 32: { QDBusPendingReply<> _r = _t->SetCursorLocation((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 33: { QDBusPendingReply<> _r = _t->SetEngine((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        case 34: { QDBusPendingReply<> _r = _t->SetSurroundingText((*reinterpret_cast< const QDBusVariant(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = _r; }  break;
        default: ;
        }
    }
}

const QMetaObjectExtraData IBusInputContextProxy::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject IBusInputContextProxy::staticMetaObject = {
    { &QDBusAbstractInterface::staticMetaObject, qt_meta_stringdata_IBusInputContextProxy,
      qt_meta_data_IBusInputContextProxy, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &IBusInputContextProxy::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *IBusInputContextProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *IBusInputContextProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_IBusInputContextProxy))
        return static_cast<void*>(const_cast< IBusInputContextProxy*>(this));
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int IBusInputContextProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void IBusInputContextProxy::CommitText(const QDBusVariant & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void IBusInputContextProxy::CursorDownLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 1, 0);
}

// SIGNAL 2
void IBusInputContextProxy::CursorUpLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 2, 0);
}

// SIGNAL 3
void IBusInputContextProxy::DeleteSurroundingText(int _t1, uint _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void IBusInputContextProxy::Disabled()
{
    QMetaObject::activate(this, &staticMetaObject, 4, 0);
}

// SIGNAL 5
void IBusInputContextProxy::Enabled()
{
    QMetaObject::activate(this, &staticMetaObject, 5, 0);
}

// SIGNAL 6
void IBusInputContextProxy::ForwardKeyEvent(uint _t1, uint _t2, uint _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void IBusInputContextProxy::HideAuxiliaryText()
{
    QMetaObject::activate(this, &staticMetaObject, 7, 0);
}

// SIGNAL 8
void IBusInputContextProxy::HideLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 8, 0);
}

// SIGNAL 9
void IBusInputContextProxy::HidePreeditText()
{
    QMetaObject::activate(this, &staticMetaObject, 9, 0);
}

// SIGNAL 10
void IBusInputContextProxy::PageDownLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 10, 0);
}

// SIGNAL 11
void IBusInputContextProxy::PageUpLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 11, 0);
}

// SIGNAL 12
void IBusInputContextProxy::RegisterProperties(const QDBusVariant & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void IBusInputContextProxy::RequireSurroundingText()
{
    QMetaObject::activate(this, &staticMetaObject, 13, 0);
}

// SIGNAL 14
void IBusInputContextProxy::ShowAuxiliaryText()
{
    QMetaObject::activate(this, &staticMetaObject, 14, 0);
}

// SIGNAL 15
void IBusInputContextProxy::ShowLookupTable()
{
    QMetaObject::activate(this, &staticMetaObject, 15, 0);
}

// SIGNAL 16
void IBusInputContextProxy::ShowPreeditText()
{
    QMetaObject::activate(this, &staticMetaObject, 16, 0);
}

// SIGNAL 17
void IBusInputContextProxy::UpdateAuxiliaryText(const QDBusVariant & _t1, bool _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void IBusInputContextProxy::UpdateLookupTable(const QDBusVariant & _t1, bool _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void IBusInputContextProxy::UpdatePreeditText(const QDBusVariant & _t1, uint _t2, bool _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void IBusInputContextProxy::UpdateProperty(const QDBusVariant & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}
QT_END_MOC_NAMESPACE
